@@ -4,6 +4,7 @@
 from logging import error
 import io
 import os
+import platform
 from pprint import pformat
 import sys
 from warnings import warn
@@ -17,11 +18,11 @@ from IPython.testing.skipdoctest import skip_doctest
 from IPython.utils.ipstruct import Struct
 
 
-class MagicsDisplay(object):
+class MagicsDisplay:
     def __init__(self, magics_manager, ignore=None):
         self.ignore = ignore if ignore else []
         self.magics_manager = magics_manager
-    
+
     def _lsmagic(self):
         """The main implementation of the %lsmagic"""
         mesc = magic_escapes['line']
@@ -39,10 +40,13 @@ class MagicsDisplay(object):
 
     def _repr_pretty_(self, p, cycle):
         p.text(self._lsmagic())
-    
+
+    def __repr__(self):
+        return self.__str__()
+
     def __str__(self):
         return self._lsmagic()
-    
+
     def _jsonable(self):
         """turn magics dict into jsonable dict of the same structure
 
@@ -59,10 +63,10 @@ class MagicsDisplay(object):
                     classname = obj.__self__.__class__.__name__
                 except AttributeError:
                     classname = 'Other'
-                
+
                 d[name] = classname
         return magic_dict
-        
+
     def _repr_json_(self):
         return self._jsonable()
 
@@ -109,12 +113,12 @@ class BasicMagics(Magics):
           Created `%%t` as an alias for `%%timeit`.
 
           In [2]: %t -n1 pass
-          1 loops, best of 3: 954 ns per loop
+          107 ns ± 43.6 ns per loop (mean ± std. dev. of 7 runs, 1 loop each)
 
           In [3]: %%t -n1
              ...: pass
              ...:
-          1 loops, best of 3: 954 ns per loop
+          107 ns ± 58.3 ns per loop (mean ± std. dev. of 7 runs, 1 loop each)
 
           In [4]: %alias_magic --cell whereami pwd
           UsageError: Cell magic function `%%pwd` not found.
@@ -122,9 +126,9 @@ class BasicMagics(Magics):
           Created `%whereami` as an alias for `%pwd`.
 
           In [6]: %whereami
-          Out[6]: u'/home/testuser'
+          Out[6]: '/home/testuser'
 
-          In [7]: %alias_magic h history "-p -l 30" --line
+          In [7]: %alias_magic h history -p "-l 30" --line
           Created `%h` as an alias for `%history -l 30`.
         """
 
@@ -296,8 +300,11 @@ Currently the magic system has the following functions:""",
 
         oname = args and args or '_'
         info = self.shell._ofind(oname)
-        if info['found']:
-            txt = (raw and str or pformat)( info['obj'] )
+        if info.found:
+            if raw:
+                txt = str(info.obj)
+            else:
+                txt = pformat(info.obj)
             page.page(txt)
         else:
             print('Object `%s` not found' % oname)
@@ -312,11 +319,7 @@ Currently the magic system has the following functions:""",
 
     @line_magic
     def colors(self, parameter_s=''):
-        """Switch color scheme for prompts, info system and exception handlers.
-
-        Currently implemented schemes: NoColor, Linux, LightBG.
-
-        Color scheme names are not case-sensitive.
+        """Switch color scheme/theme globally for IPython
 
         Examples
         --------
@@ -324,40 +327,18 @@ Currently the magic system has the following functions:""",
 
           %colors nocolor
         """
-        def color_switch_err(name):
-            warn('Error changing %s color schemes.\n%s' %
-                 (name, sys.exc_info()[1]), stacklevel=2)
 
 
-        new_scheme = parameter_s.strip()
-        if not new_scheme:
+        new_theme = parameter_s.strip()
+        if not new_theme:
+            from IPython.utils.PyColorize import theme_table
+
             raise UsageError(
-                "%colors: you must specify a color scheme. See '%colors?'")
-        # local shortcut
-        shell = self.shell
+                "%colors: you must specify a color theme. See '%colors?'."
+                f" Available themes: {list(theme_table.keys())}"
+            )
 
-        # Set shell colour scheme
-        try:
-            shell.colors = new_scheme
-            shell.refresh_style()
-        except:
-            color_switch_err('shell')
-
-        # Set exception colors
-        try:
-            shell.InteractiveTB.set_colors(scheme = new_scheme)
-            shell.SyntaxTB.set_colors(scheme = new_scheme)
-        except:
-            color_switch_err('exception')
-
-        # Set info (for 'object?') colors
-        if shell.color_info:
-            try:
-                shell.inspector.set_active_scheme(new_scheme)
-            except:
-                color_switch_err('object inspector')
-        else:
-            shell.inspector.set_active_scheme('NoColor')
+        self.shell.colors = new_theme
 
     @line_magic
     def xmode(self, parameter_s=''):
@@ -389,6 +370,7 @@ Currently the magic system has the following functions:""",
             shell.InteractiveTB.set_mode(mode=new_mode)
             print('Exception reporting mode:',shell.InteractiveTB.mode)
         except:
+            raise
             xmode_switch_err('user')
 
     @line_magic
@@ -455,7 +437,7 @@ Currently the magic system has the following functions:""",
             ptformatter.pprint = False
             disp_formatter.active_types = ['text/plain']
 
-            shell.magic('xmode Plain')
+            shell.run_line_magic("xmode", "Plain")
         else:
             # turn off
             shell.separate_in = dstore.rc_separate_in
@@ -466,7 +448,7 @@ Currently the magic system has the following functions:""",
             ptformatter.pprint = dstore.rc_pprint
             disp_formatter.active_types = dstore.rc_active_types
 
-            shell.magic('xmode ' + dstore.xmode)
+            shell.run_line_magic("xmode", dstore.xmode)
 
         # mode here is the state before we switch; switch_doctest_mode takes
         # the mode we're switching to.
@@ -490,8 +472,10 @@ Currently the magic system has the following functions:""",
         are supported:  wxPython, PyQt4, PyGTK, Tk and Cocoa (OSX)::
 
             %gui wx      # enable wxPython event loop integration
-            %gui qt4|qt  # enable PyQt4 event loop integration
-            %gui qt5     # enable PyQt5 event loop integration
+            %gui qt      # enable PyQt/PySide event loop integration
+                         # with the latest version available.
+            %gui qt6     # enable PyQt6/PySide6 event loop integration
+            %gui qt5     # enable PyQt5/PySide2 event loop integration
             %gui gtk     # enable PyGTK event loop integration
             %gui gtk3    # enable Gtk3 event loop integration
             %gui gtk4    # enable Gtk4 event loop integration
@@ -532,25 +516,25 @@ Currently the magic system has the following functions:""",
             In [1]: from math import pi
 
             In [2]: %precision 3
-            Out[2]: u'%.3f'
+            Out[2]: '%.3f'
 
             In [3]: pi
             Out[3]: 3.142
 
             In [4]: %precision %i
-            Out[4]: u'%i'
+            Out[4]: '%i'
 
             In [5]: pi
             Out[5]: 3
 
             In [6]: %precision %e
-            Out[6]: u'%e'
+            Out[6]: '%e'
 
             In [7]: pi**10
             Out[7]: 9.364805e+04
 
             In [8]: %precision
-            Out[8]: u'%r'
+            Out[8]: '%r'
 
             In [9]: pi**10
             Out[9]: 93648.047476082982
@@ -578,17 +562,86 @@ Currently the magic system has the following functions:""",
 
         cells = []
         hist = list(self.shell.history_manager.get_range())
+        outputs = self.shell.history_manager.outputs
+        exceptions = self.shell.history_manager.exceptions
+
         if(len(hist)<=1):
             raise ValueError('History is empty, cannot export')
         for session, execution_count, source in hist[:-1]:
-            cells.append(v4.new_code_cell(
-                execution_count=execution_count,
-                source=source
-            ))
-        nb = v4.new_notebook(cells=cells)
+            cell = v4.new_code_cell(execution_count=execution_count, source=source)
+            for output in outputs[execution_count]:
+                for mime_type, data in output.bundle.items():
+                    if output.output_type == "out_stream":
+                        text = data if isinstance(data, list) else [data]
+                        cell.outputs.append(v4.new_output("stream", text=text))
+                    elif output.output_type == "err_stream":
+                        text = data if isinstance(data, list) else [data]
+                        err_output = v4.new_output("stream", text=text)
+                        err_output.name = "stderr"
+                        cell.outputs.append(err_output)
+                    elif output.output_type == "execute_result":
+                        cell.outputs.append(
+                            v4.new_output(
+                                "execute_result",
+                                data={mime_type: data},
+                                execution_count=execution_count,
+                            )
+                        )
+                    elif output.output_type == "display_data":
+                        cell.outputs.append(
+                            v4.new_output(
+                                "display_data",
+                                data={mime_type: data},
+                            )
+                        )
+                    else:
+                        raise ValueError(f"Unknown output type: {output.output_type}")
+
+            # Check if this execution_count is in exceptions (current session)
+            if execution_count in exceptions:
+                cell.outputs.append(
+                    v4.new_output("error", **exceptions[execution_count])
+                )
+            cells.append(cell)
+
+        kernel_language_info = self._get_kernel_language_info()
+
+        nb = v4.new_notebook(
+            cells=cells,
+            metadata={
+                "kernelspec": {
+                    "display_name": "Python 3 (ipykernel)",
+                    "language": "python",
+                    "name": "python3",
+                },
+                "language_info": kernel_language_info
+                or {
+                    "codemirror_mode": {
+                        "name": "ipython",
+                        "version": sys.version_info[0],
+                    },
+                    "file_extension": ".py",
+                    "mimetype": "text/x-python",
+                    "name": "python",
+                    "nbconvert_exporter": "python",
+                    "pygments_lexer": "ipython3",
+                    "version": platform.python_version(),
+                },
+            },
+        )
         with io.open(outfname, "w", encoding="utf-8") as f:
             write(nb, f, version=4)
 
+    def _get_kernel_language_info(self) -> dict | None:
+        """Get language info from kernel, useful when used in Jupyter Console where kernels exist."""
+        if not hasattr(self.shell, "kernel"):
+            return
+        if not hasattr(self.shell.kernel, "language_info"):
+            return
+        if not isinstance(self.shell.kernel.language_info, dict):
+            return
+        return self.shell.kernel.language_info
+
 @magics_class
 class AsyncMagics(BasicMagics):
 
